@@ -125,6 +125,8 @@ class SklearnModel(GenericModel):
     >>> sklearn_model.predict(X_test)
     """
 
+    _PREFIX = "sklearn"
+
     def __init__(
         self,
         estimator: Callable,
@@ -544,34 +546,3 @@ class SklearnModel(GenericModel):
                 for _, col in data.iteritems()
             ]
         )
-
-    def get_data_serializer(
-        self,
-        data: Union[
-            Dict,
-            str,
-            List,
-            np.ndarray,
-            pd.core.series.Series,
-            pd.core.frame.DataFrame,
-        ],
-    ):
-        """Returns serializable input data.
-
-        Parameters
-        ----------
-        data: Union[Dict, str, list, numpy.ndarray, pd.core.series.Series,
-        pd.core.frame.DataFrame]
-            Data expected by the model deployment predict API.
-
-        Returns
-        -------
-        InputDataSerializer
-            A class containing serialized input data and original data type information.
-
-        Raises
-        ------
-        TypeError
-            if provided data type is not supported.
-        """
-        return InputDataSerializer(data)
