@@ -12,7 +12,7 @@ from typing import Dict, List
 
 from ads.common.oci_datascience import DSCNotebookSession
 from ads.opctl.backend.ads_ml_job import MLJobBackend, MLJobDistributedBackend
-from ads.opctl.backend.local import LocalBackend
+from ads.opctl.backend.local import LocalBackend, LocalBackendDistributed
 from ads.opctl.backend.ads_dataflow import DataFlowBackend
 from ads.opctl.config.base import ConfigProcessor
 from ads.opctl.config.merger import ConfigMerger
@@ -20,7 +20,15 @@ from ads.opctl.config.resolver import ConfigResolver
 from ads.opctl.config.utils import read_from_ini
 from ads.opctl.config.validator import ConfigValidator
 from ads.opctl.config.yaml_parsers import YamlSpecParser
+import fsspec
 
+from ads.opctl.distributed.cmds import (
+    update_ini,
+    increment_tag_in_ini,
+    docker_build_cmd,
+    update_image,
+    verify_and_publish_image,
+)
 from ads.opctl.constants import (
     DEFAULT_OCI_CONFIG_FILE,
     DEFAULT_PROFILE,
@@ -39,7 +47,6 @@ import yaml
 
 
 class _BackendFactory:
-
     BACKENDS_MAP = {
         "local": LocalBackend,
         "job": MLJobBackend,
@@ -77,27 +84,75 @@ def run(config: Dict, **kwargs) -> Dict:
     """
     p = ConfigProcessor(config).step(ConfigMerger, **kwargs)
     if config.get("kind") == "distributed":  # TODO: add kind factory
-
-        cluster_def = YamlSpecParser.parse_content(config)
-
-        backend = MLJobDistributedBackend(p.config)
-
-        # Define job first,
-        # Then Run
-        cluster_run_info = backend.run(
-            cluster_info=cluster_def, dry_run=p.config["execution"].get("dry_run")
+        print(
+            "......................... Initializing the process ..................................."
         )
-        if cluster_run_info:
-            cluster_run = {}
-            cluster_run["jobId"] = cluster_run_info[0].id
-            cluster_run["workDir"] = cluster_def.cluster.work_dir
-            cluster_run["mainJobRunId"] = cluster_run_info[1].id
-            cluster_run["workerJobRunIds"] = [wj.id for wj in cluster_run_info[2]]
-            yamlContent = yaml.dump(cluster_run)
-            print(yamlContent)
-            print(f"# \u2b50 To stream the logs of the main job run:")
-            print(f"# \u0024 ads opctl watch {cluster_run['mainJobRunId']}")
-        return cluster_run_info
+        ini = update_ini(
+            kwargs["tag"],
+            kwargs["registry"],
+            kwargs["dockerfile"],
+            kwargs["source_folder"],
+            config,
+            kwargs["nobuild"],
+        )
+        nobuild = kwargs["nobuild"]
+        mode = kwargs["backend"]
+        increment = kwargs["auto_increment"]
+
+        if not nobuild:
+            if increment:
+                ini = increment_tag_in_ini(ini)
+            docker_build_cmd(ini)
+        config = update_image(config, ini)
+
+        if mode == "local":
+            print(
+                "\u26A0 Docker Image: "
+                + ini.get("main", "registry")
+                + ":"
+                + ini.get("main", "tag")
+                + " is not pushed to oci artifacts."
+            )
+            print("running image: " + config["spec"]["cluster"]["spec"]["image"])
+
+            backend = LocalBackendDistributed(config)
+            backend.run()
+        elif mode == "dataflow":
+            raise RuntimeError(
+                "backend operator for distributed training can either be local or job"
+            )
+        else:
+            if not kwargs["dry_run"]:
+                verify_and_publish_image(kwargs["nopush"], config)
+
+                print("running image: " + config["spec"]["cluster"]["spec"]["image"])
+            cluster_def = YamlSpecParser.parse_content(config)
+
+            backend = MLJobDistributedBackend(p.config)
+
+            # Define job first,
+            # Then Run
+            cluster_run_info = backend.run(
+                cluster_info=cluster_def, dry_run=p.config["execution"].get("dry_run")
+            )
+            if cluster_run_info:
+                cluster_run = {}
+                cluster_run["jobId"] = cluster_run_info[0].id
+                cluster_run["workDir"] = cluster_def.cluster.work_dir
+                cluster_run["mainJobRunId"] = {
+                    cluster_run_info[1].name: cluster_run_info[1].id
+                }
+                if len(cluster_run_info[2]) > 0:
+                    cluster_run["otherJobRunIds"] = [
+                        {wj.name: wj.id} for wj in cluster_run_info[2]
+                    ]
+                yamlContent = yaml.dump(cluster_run)
+                print(yamlContent)
+                print(f"# \u2b50 To stream the logs of the main job run:")
+                print(
+                    f"# \u0024 ads opctl watch {list(cluster_run['mainJobRunId'].values())[0]}"
+                )
+            return cluster_run_info
     else:
         if p.config["execution"].get("job_id", None):
             p.config["execution"]["backend"] = "job"
@@ -110,6 +165,50 @@ def run(config: Dict, **kwargs) -> Dict:
         return _BackendFactory(p.config).backend.run()
 
 
+def run_diagnostics(config: Dict, **kwargs) -> Dict:
+    """
+    Run a job given configuration and command line args passed in (kwargs).
+
+    Parameters
+    ----------
+    config: dict
+        dictionary of configurations
+    kwargs: dict
+        keyword arguments, stores configuration from command line args
+
+    Returns
+    -------
+    Dict
+        dictionary of job id and run id in case of ML Job run, else empty if running locally
+    """
+    p = ConfigProcessor(config).step(ConfigMerger, **kwargs)
+    if config.get("kind") == "distributed":  # TODO: add kind factory
+
+        cluster_def = YamlSpecParser.parse_content(config)
+
+        backend = MLJobDistributedBackend(p.config)
+
+        # Define job first,
+        # Then Run
+        cluster_run_info = backend.run_diagnostics(
+            cluster_info=cluster_def, dry_run=p.config["execution"].get("dry_run")
+        )
+        if cluster_run_info:
+            diagnostics_report_path = os.path.join(
+                cluster_def.cluster.work_dir,
+                cluster_run_info[0].id,
+                "diagnostic_report.html",
+            )
+            with fsspec.open(
+                diagnostics_report_path, "r", **backend.oci_auth
+            ) as infile:
+                with open(kwargs["output"], "w") as outfile:
+                    outfile.write(infile.read())
+        return cluster_run_info
+    else:
+        print("Diagnostics not available for kind: {config.get('kind')}")
+
+
 def _update_env_vars(config, env_vars: List):
     """
     env_vars: List, should be formatted as [{"name": "OCI__XXX", "value": YYY},]
